@@ -160,7 +160,7 @@ class ClusterData:
 			np.savetxt(filename, self.data, delimiter=',')
 		
 		
-	def generate_data(self,add_noise_vars=False, snr=None, p_over_n=None):
+	def generate_data(self,add_noise_vars=False, snr=None, p_over_n=None,seed=None,verbose=True):
 		"""
 		Sample a dataset with clusters. The output data set possesses the
 		geometric characteristics specified by this ClusterData object.
@@ -199,6 +199,8 @@ class ClusterData:
 		p_over_n : float
 			Set the ratio between the total number of features (p) and the
 			number of samples (n). Only relevant when add_noise_vars=True.
+		seed : int or None
+			Sets the seed for random data generation, if an int is given.
 
 		Returns
 		-------
@@ -207,32 +209,33 @@ class ClusterData:
 		y : ndarray
 			Vector of cluster labels.
 		"""
-		print('Compute class sizes...')
-		self.class_sizes = self.class_bal.make_class_sizes(self)
 
-		print('Compute covariance structures...')
-		axes, sd, cov, cov_inv = self.cov_geom.make_cov(self)
+		if verbose: print('Compute class sizes...')
+		self.class_sizes = self.class_bal.make_class_sizes(self,seed=seed)
+
+		if verbose: print('Compute covariance structures...')
+		axes, sd, cov, cov_inv = self.cov_geom.make_cov(self,seed=seed)
 		self.cluster_axes = axes
 		self.cluster_sd = sd
 		self.cov = cov
 		self.cov_inv = cov_inv
 
-		print('Place cluster centers...')
-		self.centers = self.center_geom.place_centers(self)
+		if verbose: print('Place cluster centers...')
+		self.centers = self.center_geom.place_centers(self,seed=seed,verbose=verbose)
 
-		print('Sample data...')
-		self.data, self.labels = self.data_dist.sample(self)
+		if verbose: print('Sample data...')
+		self.data, self.labels = self.data_dist.sample(self,seed=seed)
 
-		print('Success!')
+		if verbose: print('Success!')
 
 		if add_noise_vars:
-			self.data = self.add_noise(self.data,snr=snr,p_over_n=p_over_n)
+			self.data = self.add_noise(self.data,snr=snr,p_over_n=p_over_n,seed=seed)
 
 		return (self.data, self.labels)
 
 
 	def add_noise(self, data, snr, p_over_n, model='unif', margin=0.1, 
-		print_warning=True):
+		print_warning=True,seed=None):
 		"""
 		Add noise features to given data.
 
@@ -257,6 +260,8 @@ class ClusterData:
 			number of samples (n).
 		model : str
 			Determine how noise features are calculated.
+		seed : int or None
+			Sets the seed for randomized data generation, if an int is given.
 
 		Returns
 		-------
@@ -264,13 +269,14 @@ class ClusterData:
 			Input data with added noise features.
 
 		"""
+		np.random.seed(seed)
 
 		n_dim = data.shape[1] 
 		n_samples = data.shape[0]
 
-		# get lower and upper feature limits
-		low = np.min(data,axis=0)
-		high = np.max(data,axis=0)
+		# get lower and upper limits for each observation across features
+		low = np.min(data,axis=1)[:,np.newaxis]
+		high = np.max(data,axis=1)[:,np.newaxis]
 
 		if (not snr) and (not p_over_n):
 			raise Exception('Must provide either snr or p_over_n to determine ' +
@@ -332,7 +338,7 @@ class CovGeom:
 								  ', or code your own algorithm for defining'
 								  ' cluster covariance structure.')
 
-	def make_orthonormal_axes(self, n_dim, n_axes):
+	def make_orthonormal_axes(self, n_dim, n_axes, seed=None):
 		"""
 		Sample orthonormal axes for cluster generation.
 		Create n_axes orthonormal axes in n_dim-dimensional space.
@@ -345,13 +351,15 @@ class CovGeom:
 			Dimensionality of the data space
 		n_axes : int
 			Number of axes to generate
+		seed : int or None
+			Sets the seed for randomized axis generation, if an int is given.
 
 		Returns
 		-------
 		out : (n_axes, n_dim) ndarray
 			Axes arranged as a matrix, each row is an axis
 		"""
-		ortho_matrix = stats.ortho_group.rvs(n_dim)
+		ortho_matrix = stats.ortho_group.rvs(n_dim,random_state=seed)
 		return ortho_matrix[:n_axes, :]
 
 
@@ -432,7 +440,7 @@ class DataDist:
 								  ' a provided implementation, e.g. GaussianDist,' +
 								  ' or code your own data distribution.')
 
-	def sample(self, clusterdata):
+	def sample(self, clusterdata, seed=None):
 		n_clusters = clusterdata.n_clusters
 		n_samples = clusterdata.n_samples
 		n_dim = clusterdata.n_dim
@@ -451,14 +459,14 @@ class DataDist:
 			# Set class label
 			y[start:end] = i
 			# Sample data
-			X[start:end,:] = self.sample_cluster(cluster_size=class_sizes[i], 
+			X[start:end,:] = self.sample_cluster(class_size=class_sizes[i], 
 										  mean=centers[i], axes=axes[i],
-										  sd=sd[i])
+										  sd=sd[i],seed=seed+i)
 			start = end
 
 		return (X, y)
 
-	def sample_cluster(self, cluster_size, mean, axes, sd):
+	def sample_cluster(self, cluster_size, mean, axes, sd, seed=None):
 		raise NotImplementedError('Cannot sample cluster from abstract DataDist. '+
 								  'Choose a provided implementation, e.g. '
 								  'GaussianDist, or code your own data distribution.')
\ No newline at end of file
